/*
 * ====================================================================
 *
 * MIT License
 * 
 * Copyright (c) [2022] 
 * [Jan Grossniklaus, Jan Kunert, Sascha Hildebrands, Kevin Hinrichs]
 * 
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy, 
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * urnished to do so, subject to the following conditions:
 * 
 * The above copyright notice and this permission notice shall
 * be included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 * 
 * ====================================================================
 */

/*
 + --------------------------------------------------------------------
 + 
 + Created    :  10/04/2022
 + Owner      :  Kevin Hinrichs
 +               
 + Description:  Driver to create the data signal for a
 +               ws2812b LED stripe.
 +               Four parallel data signals are generated.
 +
 + --------------------------------------------------------------------
 */

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include <arm_math.h>

#include "project.h" // global header file
#include "ws2812b.h"

#include "inc/hw_ints.h"
#include "inc/hw_memmap.h"
#include "inc/hw_timer.h"

#include "driverlib/rom.h"
#include "driverlib/rom_map.h"
#include "driverlib/udma.h"
#include "driverlib/gpio.h"
#include "driverlib/timer.h"

//*********************************************************************
//
// Interrupts Handler
//
//*********************************************************************
void uDMAGPIOAIntHandler(void)
{
    if ((uDMAChannelModeGet(4 | UDMA_PRI_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_PRI[0] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC5 = GPIO_PIN_5; // CH06 DMAP
        #endif
    }
    
    if ((uDMAChannelModeGet(4 | UDMA_ALT_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_ALT[0] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC6 = GPIO_PIN_6; // CH07 DMAA
        #endif
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PC5 = 0; // CH06 DMAP
    DEBUG_PC6 = 0; // CH07 DMAA
    #endif
}

void uDMAGPIOCIntHandler(void)
{
    if ((uDMAChannelModeGet(6 | UDMA_PRI_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_PRI[1] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC5 = GPIO_PIN_5; // CH06 DMAP
        #endif
    }
    
    if ((uDMAChannelModeGet(6 | UDMA_ALT_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_ALT[1] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC6 = GPIO_PIN_6; // CH07 DMAA
        #endif
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PC5 = 0; // CH06 DMAP
    DEBUG_PC6 = 0; // CH07 DMAA
    #endif
}

void uDMAGPIODIntHandler(void)
{
    if ((uDMAChannelModeGet(7 | UDMA_PRI_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_PRI[2] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC5 = GPIO_PIN_5; // CH06 DMAP
        #endif
    }
    
    if ((uDMAChannelModeGet(7 | UDMA_ALT_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_ALT[2] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC6 = GPIO_PIN_6; // CH07 DMAA
        #endif
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PC5 = 0; // CH06 DMAP
    DEBUG_PC6 = 0; // CH07 DMAA
    #endif
}

void uDMAGPIOFIntHandler(void)
{
    if ((uDMAChannelModeGet(15 | UDMA_PRI_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_PRI[3] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC5 = GPIO_PIN_5; // CH06 DMAP
        #endif
    }
    
    if ((uDMAChannelModeGet(15 | UDMA_ALT_SELECT) == UDMA_MODE_STOP))
    {
        ledStreamTranferState_ALT[3] = EMPTY;
        
        #ifndef DEBUG_OFF
        DEBUG_PC6 = GPIO_PIN_6; // CH07 DMAA
        #endif
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PC5 = 0; // CH06 DMAP
    DEBUG_PC6 = 0; // CH07 DMAA
    #endif
}

void WTIMER5BIntHandler(void)
{
    TimerIntClear(WTIMER5_BASE, TIMER_TIMB_TIMEOUT);
    
    // reset time for led is done
    ledResetTimeIsRunnig = 0;
}

//*********************************************************************
//
// Functions
//
//*********************************************************************
void startTimer()
{
    // start timers with a delay to prevent signal conflicts
    // use direct memory access instead of the tivaware
    
    // TIMER 0
    TimerEnable(TIMER0_BASE, TIMER_BOTH);
    for (int i = 0; i < 3; i++)
    {
        while ((*((volatile uint32_t *)0x40030054)) <= 15)
        {
            // ...waiting the period is start
        }
        
        while ((*((volatile uint32_t *)0x40030054)) > 15)
        {
            // ...waiting the period is done
        }
    }
    MAP_SysCtlDelay(1);
    
    // TIMER 1
    TimerEnable(TIMER1_BASE, TIMER_BOTH);
    for (int i = 0; i < 3; i++)
    {
        while ((*((volatile uint32_t *)0x40031054)) <= 15)
        {
            // ...waiting the period is start
        }
        
        while ((*((volatile uint32_t *)0x40031054)) > 15)
        {
            // ...waiting the period is done
        }
    }
    MAP_SysCtlDelay(1);
    
    // TIMER 2
    TimerEnable(TIMER2_BASE, TIMER_BOTH);
    for (int i = 0; i < 3; i++)
    {
        while ((*((volatile uint32_t *)0x40032054)) <= 15)
        {
            // ...waiting the period is start
        }
        
        while ((*((volatile uint32_t *)0x40032054)) > 15)
        {
            // ...waiting the period is done
        }
    }
    MAP_SysCtlDelay(1);
    
    // TIMER 3
    TimerEnable(TIMER3_BASE, TIMER_BOTH);
}

void stopTimer()
{
    // stop all timers when the signal is off
    // use direct memory access instead of the tivaware
    
    // Timer 0A
    while ((*((volatile uint32_t *)0x40030050)) > SIGNAL_RESET)
    {
        // ...waiting the signal is off
    }
    (*((volatile uint32_t *)0x4003000C)) &= ~0x00000001;
    
    // Timer 1A
    while ((*((volatile uint32_t *)0x40031050)) > SIGNAL_RESET)
    {
        // ...waiting the signal is off
    }
    (*((volatile uint32_t *)0x4003100C)) &= ~0x00000001;
    
    // Timer 2A
    while ((*((volatile uint32_t *)0x40032050)) > SIGNAL_RESET)
    {
        // ...waiting the signal is 0
    }
    (*((volatile uint32_t *)0x4003200C)) &= ~0x00000001;
    
    // Timer 3A
    while ((*((volatile uint32_t *)0x40033050)) > SIGNAL_RESET)
    {
        // ...waiting the signal is 0
    }
    (*((volatile uint32_t *)0x4003300C)) &= ~0x00000001;
    
    // Timer 0B
    while ((*((volatile uint32_t *)0x40030054)) > SIGNAL_DMA)
    {
        // ...waiting the signal is off
    }
    (*((volatile uint32_t *)0x4003000C)) &= ~0x00000100;
    
    // Timer 1B
    while ((*((volatile uint32_t *)0x40031054)) > SIGNAL_DMA)
    {
        // ...waiting the signal is off
    }
    (*((volatile uint32_t *)0x4003100C)) &= ~0x00000100;
    
    // Timer 2B
    while ((*((volatile uint32_t *)0x40032054)) > SIGNAL_DMA)
    {
        // ...waiting the signal is 0
    }
    (*((volatile uint32_t *)0x4003200C)) &= ~0x00000100;
    
    // Timer 3B
    while ((*((volatile uint32_t *)0x40033054)) > SIGNAL_DMA)
    {
        // ...waiting the signal is 0
    }
    (*((volatile uint32_t *)0x4003300C)) &= ~0x00000100;
    
    /*
    // Timer 0
    while ((*((volatile uint32_t *)0x40030050)) > SIGNAL_DMA)
    {
        // ...waiting the signal is off
    }
    TimerDisable(TIMER0_BASE, TIMER_BOTH);
    
    // Timer 1
    while ((*((volatile uint32_t *)0x40031050)) > SIGNAL_DMA)
    {
        // ...waiting the signal is off
    }
    TimerDisable(TIMER1_BASE, TIMER_BOTH);
    
    // Timer 2A
    while ((*((volatile uint32_t *)0x40032050)) > SIGNAL_DMA)
    {
        // ...waiting the signal is 0
    }
    TimerDisable(TIMER2_BASE, TIMER_BOTH);
    
    // Timer 3
    while ((*((volatile uint32_t *)0x40033050)) > SIGNAL_DMA)
    {
        // ...waiting the signal is 0
    }
    TimerDisable(TIMER3_BASE, TIMER_BOTH);
    */
    
    
    // reset all counters
    (*((volatile uint32_t *)0x40030050)) = 0x00000000;
    (*((volatile uint32_t *)0x40030054)) = 0x00000000;
    (*((volatile uint32_t *)0x40031050)) = 0x00000000;
    (*((volatile uint32_t *)0x40031054)) = 0x00000000;
    (*((volatile uint32_t *)0x40032050)) = 0x00000000;
    (*((volatile uint32_t *)0x40032054)) = 0x00000000;
    (*((volatile uint32_t *)0x40033050)) = 0x00000000;
    (*((volatile uint32_t *)0x40033054)) = 0x00000000;
}

void updatePixelData(streamDataDataTyp* streamData, uint16_t offsetNum, 
                     uint8_t red, uint8_t green, uint8_t blue)
{
    streamData->pixelData[offsetNum][0] = green;
    streamData->pixelData[offsetNum][1] = red;
    streamData->pixelData[offsetNum][2] = blue;
}

void convertStreamData0PRI1Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks <----
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 0; i < 29; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[0][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[0][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData1PRI1Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks <----
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 116; i < 145; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[1][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[1][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData2PRI1Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks <----
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 232; i < 261; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[2][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[2][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData3PRI1Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks <----
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 348; i < 377; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[3][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[3][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData0ALT2Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks <----
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 29; i < 58; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[0][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[0][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData1ALT2Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks <----
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 145; i < 174; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[1][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[1][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData2ALT2Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks <----
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 261; i < 290; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[2][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[2][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData3ALT2Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks <----
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 377; i < 406; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[3][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[3][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData0PRI3Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks <----
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 58; i < 87; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[0][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[0][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData1PRI3Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks <----
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 174; i < 203; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[1][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[1][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData2PRI3Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks <----
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 290; i < 319; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[2][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[2][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData3PRI3Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks <----
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 406; i < 435; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_PRI[3][dma_index] = 30;
            }
            else
            {
                streamData->streamData_PRI[3][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData0ALT4Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks <----
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 87; i < 116; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[0][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[0][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    // put reset code and the end
    streamData->streamData_ALT[0][dma_index] = 98;
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData1ALT4Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks <----
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 203; i < 232; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[1][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[1][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    // put reset code and the end
    streamData->streamData_ALT[1][dma_index] = 98;
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData2ALT4Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks <----
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 319; i < 348; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[2][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[2][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    // put reset code and the end
    streamData->streamData_ALT[2][dma_index] = 98;
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void convertStreamData3ALT4Turn(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA4 = GPIO_PIN_4; // CH02 Conv
    #endif
    
    uint32_t datablock;
    uint16_t dma_index;
    
    // convert every bit from a datablock (match register timer)
    //
    // i = datablock size
    // j = streamData index
    //
    
    // set dma_index
    dma_index = 0;
    
    // 1.Turn
    // Data0 000-028 Datablocks + 000 Offset = 000-028 Datablocks
    // Data1 000-028 Datablocks + 116 Offset = 116-144 Datablocks
    // Data2 000-028 Datablocks + 232 Offset = 232-260 Datablocks
    // Data3 000-028 Datablocks + 348 Offset = 348-376 Datablocks
    
    // 2.Turn
    // Data0 029-057 Datablocks + 000 Offset = 029-057 Datablocks
    // Data1 029-057 Datablocks + 116 Offset = 145-173 Datablocks
    // Data2 029-057 Datablocks + 232 Offset = 261-289 Datablocks
    // Data3 029-057 Datablocks + 348 Offset = 377-405 Datablocks
    
    // 3.Turn
    // Data0 058-086 Datablocks + 000 Offset = 058-086 Datablocks
    // Data1 058-086 Datablocks + 116 Offset = 174-202 Datablocks
    // Data2 058-086 Datablocks + 232 Offset = 290-318 Datablocks
    // Data3 058-086 Datablocks + 348 Offset = 406-434 Datablocks
    
    // 4.Turn (+ RESET CODE)
    // Data0 087-115 Datablocks + 000 Offset = 087-115 Datablocks
    // Data1 087-115 Datablocks + 116 Offset = 203-231 Datablocks
    // Data2 087-115 Datablocks + 232 Offset = 319-347 Datablocks
    // Data3 087-115 Datablocks + 348 Offset = 435-463 Datablocks <----
    //                                               ^
    //                                               |
    //                       for the for loop + 1 ---+
    //
    
    for (int i = 435; i < 464; i++)
    {
        datablock = ((streamData->pixelData[i][0] << 16) | 
                     (streamData->pixelData[i][1] << 8) | 
                     (streamData->pixelData[i][2]));
        
        for (int j = (24 - 1); j >= 0; j--)
        {
            if (datablock & (1 << j))
            {
                streamData->streamData_ALT[3][dma_index] = 30;
            }
            else
            {
                streamData->streamData_ALT[3][dma_index] = 70;
            }
            dma_index++;
        }
    }
    
    // put reset code and the end
    streamData->streamData_ALT[3][dma_index] = 98;
    
    #ifndef DEBUG_OFF
    DEBUG_PA4 = 0; // CH02 Conv
    #endif
}

void startStreamPri(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA5 = GPIO_PIN_5; // CH03 Pri
    #endif
    
    MAP_uDMAChannelTransferSet(4 | UDMA_PRI_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_PRI[0], 
                               (void *)(TIMER0_BASE + TIMER_O_TAMATCHR), 
                               696);
        
    MAP_uDMAChannelTransferSet(6 | UDMA_PRI_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_PRI[1], 
                               (void *)(TIMER1_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelTransferSet(7 | UDMA_PRI_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_PRI[2], 
                               (void *)(TIMER2_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelTransferSet(15 | UDMA_PRI_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_PRI[3], 
                               (void *)(TIMER3_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelEnable(4 | UDMA_PRI_SELECT);
    MAP_uDMAChannelEnable(6 | UDMA_PRI_SELECT);
    MAP_uDMAChannelEnable(7 | UDMA_PRI_SELECT);
    MAP_uDMAChannelEnable(15 | UDMA_PRI_SELECT);
    
    for (int i = 0; i < 4; i++)
    {
        ledStreamTranferState_PRI[i] = FULL;
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA5 = 0; // CH03 Pri
    #endif
}

void startStreamAlt(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA6 = GPIO_PIN_6; // CH04 Alt
    #endif
    
    MAP_uDMAChannelTransferSet(4 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[0], 
                               (void *)(TIMER0_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelTransferSet(6 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[1], 
                               (void *)(TIMER1_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelTransferSet(7 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[2], 
                               (void *)(TIMER2_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelTransferSet(15 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[3], 
                               (void *)(TIMER3_BASE + TIMER_O_TAMATCHR), 
                               696);
    
    MAP_uDMAChannelEnable(4 | UDMA_ALT_SELECT);
    MAP_uDMAChannelEnable(6 | UDMA_ALT_SELECT);
    MAP_uDMAChannelEnable(7 | UDMA_ALT_SELECT);
    MAP_uDMAChannelEnable(15 | UDMA_ALT_SELECT);

    for (int i = 0; i < 4; i++)
    {
        ledStreamTranferState_ALT[i] = FULL;
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA6 = 0; // CH04 Alt
    #endif
}

void startStreamAltReset(streamDataDataTyp* streamData)
{
    #ifndef DEBUG_OFF
    DEBUG_PA6 = GPIO_PIN_6; // CH04 Alt
    #endif
    
    MAP_uDMAChannelTransferSet(4 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[0], 
                               (void *)(TIMER0_BASE + TIMER_O_TAMATCHR), 
                               697);
    
    MAP_uDMAChannelTransferSet(6 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[1], 
                               (void *)(TIMER1_BASE + TIMER_O_TAMATCHR), 
                               697);
    
    MAP_uDMAChannelTransferSet(7 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[2], 
                               (void *)(TIMER2_BASE + TIMER_O_TAMATCHR), 
                               697);
    
    MAP_uDMAChannelTransferSet(15 | UDMA_ALT_SELECT, 
                               UDMA_MODE_PINGPONG, &streamData->streamData_ALT[3], 
                               (void *)(TIMER3_BASE + TIMER_O_TAMATCHR), 
                               697);
    
    MAP_uDMAChannelEnable(4 | UDMA_ALT_SELECT);
    MAP_uDMAChannelEnable(6 | UDMA_ALT_SELECT);
    MAP_uDMAChannelEnable(7 | UDMA_ALT_SELECT);
    MAP_uDMAChannelEnable(15 | UDMA_ALT_SELECT);

    for (int i = 0; i < 4; i++)
    {
        ledStreamTranferState_ALT[i] = FULL;
    }
    
    #ifndef DEBUG_OFF
    DEBUG_PA6 = 0; // CH04 Alt
    #endif
}

uint8_t sendLED(streamDataDataTyp* streamData)
{
    static uint8_t steps = 0;
    uint8_t retValue;
    
    // sending finished
    retValue = 0x00;
    
    switch (steps)
    {
        case 0:
        
        #ifndef DEBUG_OFF
        DEBUG_PA2 = GPIO_PIN_2; // CH00 Start LED
        #endif
        
        //wait reset time befor start again
        if (!ledResetTimeIsRunnig)
        {
            steps = 1;
        }
        break;
        
        case 1:
        // convert 1.Turn
        convertStreamData0PRI1Turn(streamData);
        convertStreamData1PRI1Turn(streamData);
        convertStreamData2PRI1Turn(streamData);
        convertStreamData3PRI1Turn(streamData);
        
        // start 1.Turn
        startStreamPri(streamData);
        
        // start timer for led stream
        startTimer();
        
        steps = 2;
        break;
        
        case 2:
        // convert 2.Turn
        convertStreamData0ALT2Turn(streamData);
        convertStreamData1ALT2Turn(streamData);
        convertStreamData2ALT2Turn(streamData);
        convertStreamData3ALT2Turn(streamData);
        
        // start 2.Turn
        startStreamAlt(streamData);

        steps = 3;
        break;
        
        case 3:
        // wait 1.Turn is finished
        if (ledStreamTranferState_PRI[3] == EMPTY &&
            ledStreamTranferState_PRI[2] == EMPTY &&
            ledStreamTranferState_PRI[1] == EMPTY &&
            ledStreamTranferState_PRI[0] == EMPTY)
        {
            // convert 3.Turn
            convertStreamData0PRI3Turn(streamData);
            convertStreamData1PRI3Turn(streamData);
            convertStreamData2PRI3Turn(streamData);
            convertStreamData3PRI3Turn(streamData);
            
            // start 3.Turn
            startStreamPri(streamData);
            
            steps = 4;
        }
        break;
        
        case 4:
        // wait 3.Turn is finished
        if (ledStreamTranferState_ALT[3] == EMPTY &&
            ledStreamTranferState_ALT[2] == EMPTY &&
            ledStreamTranferState_ALT[1] == EMPTY &&
            ledStreamTranferState_ALT[0] == EMPTY)
        {
            // convert 4.Turn + Reset Code
            convertStreamData0ALT4Turn(streamData);
            convertStreamData1ALT4Turn(streamData);
            convertStreamData2ALT4Turn(streamData);
            convertStreamData3ALT4Turn(streamData);
            
            // start 4.Turn + Reset Code
            startStreamAltReset(streamData);
            
            steps = 5;
        }
        break;
        
        case 5:
        // wait 4.Turn is finished
        if (ledStreamTranferState_ALT[3] == EMPTY &&
            ledStreamTranferState_ALT[2] == EMPTY &&
            ledStreamTranferState_ALT[1] == EMPTY &&
            ledStreamTranferState_ALT[0] == EMPTY)
        {
            // sending is finished
            retValue = 0x1;
            
            // stop timers for the led stream
            stopTimer();
            
            // start reset code time
            ledResetTimeIsRunnig = 1;
            TimerEnable(WTIMER5_BASE, TIMER_B);
            
            #ifndef DEBUG_OFF
            DEBUG_PA2 = 0; // CH00 Start LED
            #endif
            
            steps = 0;
        }
        break;
        
        default:
        // ...error
        // RED LED from board have to be blinking
        // comes later
        break;
    }
    return retValue;
}